@@ -1,10 +1,10 @@
 /* Openoffice.org-voikko: Finnish linguistic extension for OpenOffice.org
- * Copyright (C) 2007 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2007 - 2008 Harri Pitkänen <hatapitk@iki.fi>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +12,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *********************************************************************************/
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ ***************************************************************************/
 
 #include <rtl/ustrbuf.hxx>
 #include <libvoikko/voikko.h>
@@ -129,10 +128,14 @@ uno::Reference<linguistic2::XPossibleHyphens> SAL_CALL
 	VOIKKO_DEBUG("Hyphenator::createPossibleHyphens");
 	if (!voikko_initialized) return 0;
 	if (aWord.getLength() > 10000) return 0;
+	sal_Int16 len = (sal_Int16) aWord.getLength();
 	thePropertyManager->setValues(aProperties);
 
 	// If the word is too short to be hyphenated, return no hyphenation points
-	if (aWord.getLength() < thePropertyManager->getHyphMinWordLength()) {
+	sal_Int16 minLeading = thePropertyManager->getHyphMinLeading();
+	sal_Int16 minTrailing = thePropertyManager->getHyphMinTrailing();
+	if (len < thePropertyManager->getHyphMinWordLength() ||
+	    len < minLeading + minTrailing) {
 		thePropertyManager->resetValues(aProperties);
 		return 0;
 	}
@@ -145,27 +148,18 @@ uno::Reference<linguistic2::XPossibleHyphens> SAL_CALL
 		return 0;
 	}
 
-	/* Count the number of hyphenation points that do not correspond
-	 * to a real hyphen in the word. */
-	sal_Int16 hpcount = 0;
-	for (sal_Int16 i = 0; i < aWord.getLength(); i++) {
-		if (hyphenationPoints[i] == '-') {
-			hpcount++;
-		}
-	}
-
-	uno::Sequence<sal_Int16> hyphenSeq(hpcount);
-	sal_Int16 *pPos = hyphenSeq.getArray();
+	uno::Sequence<sal_Int16> hyphenSeq(0);
 	OUStringBuffer hyphenatedWordBuffer;
 	OUString hyphenatedWord;
 	sal_Int16 nHyphCount = 0;
 
-	for (sal_Int16 i = 0; i < aWord.getLength(); i++) {
+	for (sal_Int16 i = 0; i < len; i++) {
 		hyphenatedWordBuffer.append(aWord[i]);
-		if (hyphenationPoints[i + 1] == '-') {
-			pPos[nHyphCount] = i;
+		if (i >= minLeading - 1 && i < len - minTrailing &&
+		    hyphenationPoints[i + 1] == '-') {
+			hyphenSeq.realloc(nHyphCount + 1);
+			hyphenSeq[nHyphCount++] = i;
 			hyphenatedWordBuffer.append(sal_Unicode('='));
-			nHyphCount++;
 		}
 	}
 
